Unit
  DBServ;

{$N+,V- Data Base Files Service.
  Copyright (C) Kulimov V. 1994-95.
  Portions Copyright (C) Joint-Stick Commercial Bank "Komibank".
  All Rights Reserved. }

Interface

Uses
  Objects, IniServ, Crt, Dialogs;

Type
  PDBCollection = ^TDBCollection;
  TDBCollection = Object(TCollection)
    WhatAboutExit: Boolean;
    Procedure Error(Code, Info: Integer); virtual;
  End;

  TFieldType = ( ftCharacter, ftDate, ftLogical,
                ftNumeric, ftMemo, ftFloat );

  TDateStr = String[8]; { YYYYMMDD }

  TBuffer  = Array[0..1024] of Char;

Const
  TFieldStr: Array[TFieldType] of String[9] =
                ('Character', 'Date', 'Logical',
                  'Numeric', 'Memo', 'Float' );
  stNoDataBase = -8;

Type
  PField = ^TField;
  TField = Object(TObject)
    Name: String[11];
    What: TFieldType;
                { C -   Character,
                  D -   Date,     
                  L -   Logical,  
                  N -   Numeric,  
                  M -   Memo,     
                  F -   Float.     }
    Offs, Length, Decimal: Byte;
    TagMDX         : Boolean;
  End;

  PDbFile = ^TDbFile;
  TDbFile = Object(TBufStream)
    StartPos,
    RecSize,
    FieldsCount : Integer;
    RecCount,
    CurrentRec  : LongInt;
    Fields      : PCollection;
    FName       : FNameStr;
    ModifyDate  : String[10];
    Constructor Init(FileName: FNameStr; Mode, Size: Word);
    Function  BuildFieldsCollection: PCollection;
    Function  GetRecSize: Integer;
    Function  GetRecCount: LongInt;
{   Function  SeekRec(Pos: LongInt): Boolean;
    Function  Skip(Count: LongInt): Boolean; }
    Function  GetCurrentRec: LongInt;
    Function  ReadRec(Var Rec): Boolean;
    Function  WriteRec(Var Rec): Boolean;
    Function  BuildRecCollection: PCollection; Virtual;
    Function  GetHeader(Pos: Integer): String; Virtual;
    Procedure WriteFields;
    Procedure GetInfo;
    Function  ToBinary(BinName: FNameStr): Boolean;
    Procedure WriteHeader;
    Procedure MakeText(C: PCollection);
  End;


  PFieldsListBox = ^TFieldsListBox;
  TFieldsListBox = Object(TListBox)
    Function GetText(Item: Integer; MaxLen: Integer): String; virtual;
  End;


Procedure WaitPlease;
Procedure DeleteWait;
Procedure ToDate(Var D: String);

Implementation

Uses
  Views, App, MsgBox, Memory, Drivers, Dos, Tools;

Function Space(P: Byte): String;
Var
  i: Byte;
  S: String;
Begin
  S := '';
  For i := 1 to P do
    S := S + ' ';
  Space := S;
End;

Const
  DateBreaker = '-';

Procedure ToDate(Var D: String);
Begin
  D := Copy(D, 7, 2) + DateBreaker +
       Copy(D, 5, 2) + DateBreaker +
       Copy(D, 1, 4);
End;

Const
  WaitWindow: PDialog = Nil;

Procedure WaitPlease;
Var
  R: TRect;
Begin
  if Not Assigned(WaitWindow) then
  Begin
    R.Assign(0, 0, 38, 5);
    WaitWindow := New(PDialog, Init(R, ''));
    With WaitWindow^ do
    Begin
      Options := Options or ofCentered;
      Flags := Flags And (Not wfClose) And (Not wfMove);
      R.Grow(-1, -2);
      Insert(New(PStaticText, Init(R, ^C'Wait, please...')));
    End;
    DeskTop^.Insert(WaitWindow);
  End;
End;

Procedure DeleteWait;
Begin
  if Assigned(WaitWindow) then
     Dispose(WaitWindow, Done);
End;

{  TDBColection Object }

Procedure TDBCollection.Error;
Begin
  DeleteWait;
  MessageBox(^C'  !', Nil, mfError + mfOkButton);
  WhatAboutExit := True;
End;

{  TDBFile Object  }

Constructor TDBFile.Init(FileName: FNameStr; Mode, Size: Word);
Var
  B: Byte;
  S,P: String;
Begin
  Inherited Init(FileName, Mode, Size);
  if Status <> stOk then
     Exit;
  Seek(1);
  Read(B, 1);
  if (B > 99) or (B = 0) then
  Begin
     Status := stNoDataBase;
     Exit;
  End;
  Str(B, P);
  While Length(P) < 2 do
    P := '0' + P;
  P := '19' + P;
  Read(B, 1);
  if (B > 12) or (B = 0) then
  Begin
     Status := stNoDataBase;
     Exit;
  End;
  S := P;
  Str(B, P);
  While Length(P) < 2 do
    P := '0' + P;
  S := P + DateBreaker + S;
  Read(B, 1);
  if (B > 31) or (B = 0) then
  Begin
     Status := stNoDataBase;
     Exit;
  End;
  Str(B, P);
  While Length(P) < 2 do
    P := '0' + P;
  S := P + DateBreaker + S;

  ModifyDate := S;
  Read(RecCount, 4);
  Read(StartPos, 2);
  Read(RecSize, 2);
  FieldsCount := 0;
  Fields := BuildFieldsCollection;
  if Not Assigned(Fields) then
     Status := stError;
  CurrentRec := 0;
  FName := FileName;
End;

Function TDBFile.BuildFieldsCollection: PCollection;
Var
  C   : PCollection;
  Buf : Array[0..31] of Char;
  F   : PField;
  S   : String;
  i,j : Integer;
Begin
  C := New(PCollection, Init(5, 2));
  Seek(32);
  j := 0;
  Read(Buf, 32);
  if Status <> stOk then
  Begin
    Dispose(C, Done);
    C := Nil;
  End
  else
  While Buf[0] <> #13 do
  Begin
    F := New(PField, Init);
    S := Copy(Buf, 0, 10);
    S[0] := Char(Pos(#0,S)-1);
{    if Length(S) < 11 then S := S + Space(11-Length(S));}
    F^.Name := S;
    Case Buf[11] of
      'C': F^.What := ftCharacter;
      'D': F^.What := ftDate;
      'L': F^.What := ftLogical;
      'N': F^.What := ftNumeric;
      'M': F^.What := ftMemo;
      'F': F^.What := ftFloat
    End;
    F^.Offs := j;
    F^.Length := Byte(Buf[16]);
    j := j + F^.Length;
    F^.Decimal := Byte(Buf[17]);
    F^.TagMDX := Buf[31] <> #0;
    C^.Insert(F);
    Inc(FieldsCount);
    Read(Buf, 32);
  End;
  BuildFieldsCollection := C;
End;

Function TDBFile.GetRecSize: Integer;
Begin
  GetRecSize := RecSize;
End;

Function TDBFile.GetRecCount: LongInt;
Begin
  GetRecCount := RecCount;
End;

Function TDBFile.GetCurrentRec: LongInt;
Begin
  GetCurrentRec := CurrentRec;
End;

Function TDBFile.ReadRec(Var Rec): Boolean;
Begin
  Read(Rec, RecSize);
  ReadRec := Status = stOk;
End;

Function TDBFile.WriteRec(Var Rec): Boolean;
Begin
  Write(Rec, RecSize);
  WriteRec := Status = stOk;
End;

Procedure TDbFile.WriteFields;
Var
  i: Integer;
  Procedure W(Item: Pointer); Far;
  Begin
    With PField(Item)^ do
       WriteLn(Name, ' ', TFieldStr[What], ' ', Length:3, ' ', Decimal:3, ' ', Offs);
  End;
Begin
  Fields^.ForEach(@W);
End;

Function TDBFile.GetHeader(Pos: Integer): String;
Var
  S: String;
  i: Byte;
Begin
  S := 'Del'#179;
  For i := 0 to Fields^.Count - 1 do
  With PField(Fields^.At(i))^ do
  Begin
    if System.Length(Name) < Length then
       S := S + Space(Length - System.Length(Name));
    if What = ftDate then S := S + '  ';
    S := S + Name + #179;
  End;
  Dec(S[0]);
  GetHeader := S;
End;

Function TDBFile.BuildRecCollection: PCollection;
Var
  C: PDBCollection;
  S: String;
  Buf: Array[0..254] of Char;

  Function BuildNormalRec: String;
  Var
    P,D: String;
    i,j: Byte;
    Procedure ToStr(Item: Pointer);
    Begin
      With PField(Item)^ do
      Begin
        if Length < System.Length(Name) then
           P := P + Space(System.Length(Name) - Length);
        D := Copy(Buf, i, Length);
        if What = ftDate then ToDate(D);
        P := P + D + #179;
        Inc(i, Length);
      End;
    End;
  Begin
    P := Buf[0] + #$20#179;
    i := 2;
    With Fields^ do
    For j := 0 to Count - 1 do
      ToStr(At(j));
    Dec(P[0]);
    BuildNormalRec := P;
  End;

Begin
  C := New(PDBCollection, Init(RecCount, 0));
  C^.WhatAboutExit := False;
  Seek(StartPos);
  While True do
  Begin
    if Not ReadRec(Buf) then
       Break;
    S := BuildNormalRec;
    C^.Insert(NewStr(S));
    if C^.WhatAboutExit then
    Begin
      ClrScr;
      WriteLn('.'#7#46#7#46#7#46#7#46);
      Halt;
    End;
  End;
  BuildRecCollection := C;
  Seek(0);
End;

Procedure TDBFile.GetInfo;
Var
  R,B: TRect;
  D: PDialog;
  V: PView;
  SB: PScrollBar;
  S, F, ResultStr: String;
  P: Pointer;
Begin
  R.Assign(0, 0, 74, 21);
  D := New(PDialog, Init(R, 'File information'));
  With D^ do
  Begin
    Options := Options or ofCentered;

    { get filename }
    S := ' 䠩: %-64s';
    F := FExpand(FName);
    if F[0] > #64 then
    Begin
      System.Insert('...', F, 3);
      While Length(F) > 64 do
        System.Delete(F, 6, 1);
    End;
    P := @F;
    FormatStr(ResultStr, S, P);
    R.Assign(2, 1, 73, 2);
    Insert(New(PStaticText, Init(R, ResultStr)));

    R.Move(0, 1);
    S := ' ᫥ 䨪樨: %-10s';
    P := @ModifyDate;
    FormatStr(ResultStr, S, P);
    Insert(New(PStaticText, Init(R, ResultStr)));

    R.Move(0, 1);
    Str(RecCount, F);
    S := '᫮ ᥩ: %-10s';
    P := @F;
    FormatStr(ResultStr, S, P);
    Insert(New(PStaticText, Init(R, ResultStr)));

    R.Move(0, 1);
    Str(RecSize, F);
    S := ' : %s(bytes)';
    P := @F;
    FormatStr(ResultStr, S, P);
    Insert(New(PStaticText, Init(R, ResultStr)));

    R.Move(0, 1);
    Str(StartPos, F);
    S := ' : %s(bytes)';
    P := @F;
    FormatStr(ResultStr, S, P);
    Insert(New(PStaticText, Init(R, ResultStr)));

    R.Move(0, 1);
    Str(FieldsCount, F);
    S := '᫮ : %-10s';
    P := @F;
    FormatStr(ResultStr, S, P);
    Insert(New(PStaticText, Init(R, ResultStr)));

    R.Move(0, 1);
    Insert(New(PStaticText, Init(R,
    '  #                   筠  ')));

    R.Move(0, 1);
    Inc(R.B.Y, 8);
    B.Copy(R);
    R.A.X := R.B.X - 1;
    SB := New(PScrollBar, Init(R));
    Insert(SB);

    R.Copy(B);
    R.Move(-1, 0);
    V := New(PFieldsListBox, Init(R, 1, SB));
    Insert(V);
    PFieldsListBox(V)^.NewList(Fields);

    R.Assign(0, 18, 12, 20);
    V := New(PButton, Init(R, '~O~k', cmOk, bfDefault));
    V^.Options := V^.Options or ofCenterX;
    Insert(V);

    SelectNext(False);
  End;
  Application^.ExecuteDialog(D, Nil);
End;

Function TDBFile.ToBinary(BinName: FNameStr): Boolean;
Var
  Buf: TBuffer;
  F  : PBufStream;

  Procedure WriteNormalRec;
  Var
    D        : String;
    i        : Word;
    j        : Byte;
    xLongInt : LongInt;
    xDouble  : Double;

    Function LongCopy(Source: TBuffer; Pos: Word; Count: Byte): String;
    Var
      i: Word;
      E: String;
    Begin
      E := '';
      for i := Pos to Pos + Count - 1 do
        E := E + Source[i];
      LongCopy := E;
    End;

    Procedure WriteField(Item: Pointer);
    Var
      Code: Integer;
      D   : String;
    Begin
      With PField(Item)^ do
      Begin
        D := LongCopy(Buf, i, Length);
        Case What of
          ftDate,
          ftCharacter: F^.Write(D, Length + 1);
          ftNumeric: Begin
                       Val(D, xLongInt, Code);
                       if Code <> 0 then
                          xLongInt := 0;
                       F^.Write(xLongInt, SizeOf(LongInt))
                     End;
          ftFloat: Begin
                       Val(D, xDouble, Code);
                       if Code <> 0 then
                          xDouble := 0;
                       F^.Write(xDouble, SizeOf(Double))
                     End;
          ftLogical: ;
        End;
        Inc(i, Length);
      End;
    End;

  Begin
    i := 1;
    With Fields^ do
    For j := 0 to Count - 1 do
        WriteField(At(j));
{    WriteLn; ReadLn;}
  End;

Begin
  ToBinary := True;
  F := New(PBufStream, Init(BinName, stCreate, 32768));
  if F^.Status <> stOk then
  Begin
    Dispose(F, Done);
    ToBinary := False;
    Exit;
  End;
  Status := stOk;
  Seek(StartPos);
  While ReadRec(Buf) do
    WriteNormalRec;
  Dispose(F, Done);
End;

Procedure TDBFile.MakeText(C: PCollection);
Var
  Ch: Char;

  Function MakeFucke(S: String): String;
  Var
    P: String;
    i: Byte;
  Begin
    P := '';
    For i := 1 to Byte(S[0]) do
    if S[i] = '' then
      P := P + Ch
    else
      P := P + S[i];
    MakeFucke := P
  End;

Var
  D: PKeyEnterDialog;
  R: TRect;
  N: String[50];
  I: PInputLine;
  T: Text;
  P: Pointer;
  S: String;
  j: LongInt;

Begin
  R.Assign(0, 0, 70, 7);
  D := New(PKeyEnterDialog, Init(R, ' ⥪⮢ 䠩'));
  With D^ do
  Begin
    Options := Options or ofCentered;

    R.Assign(15, 2, 67, 3);
    I := New(PInputLine, Init(R, 50));
    Insert(I);
    R.Assign(1, 2, 14, 3);
    Insert(New(PLabel, Init(R, ' 䠩', I)));

    R.Assign(39, 4, 49, 6);
    OkButton := New(PButton, Init(R, '~O~k', cmOk, bfNormal));
    Insert(OkButton);

    R.Assign(52, 4, 62, 6);
    CancelButton := New(PButton, Init(R, '~C~ancel', cmCancel, bfNormal));
    Insert(CancelButton);
    SelectNext(False);
  End;

  N := '';

  if Application^.ExecuteDialog(D, @N) = cmOk then
  Begin
    if Exist(N) then
    Begin
      P := @N;
      if MessageBox(^C' %s . ९?', @P, mfYesNo + mfConfirmation) <> cmYes then
         Exit;
    End;
    S := GetProfileString('System', 'Divide', '" "');
    Ch := S[2];
 {   ShowStatusBox('Wait...'); }
    Assign(T, N);
    ReWrite(T);
    S := GetHeader(0);
    WriteLn(T, MakeFucke(S));
    For j := 0 to C^.Count - 1 do
    Begin
      if j mod 66 = 0 then
      Begin
        WriteLn(T);
        WriteLn(T);
        WriteLn(T, ' ', j div 66 + 1);
        WriteLn(T);
      End;
      WriteLn(T, j:6, MakeFucke(PString(C^.At(j))^));
    End;
    Close(T);
{    KillStatusBox;}
  End;
End;

Procedure TDBFile.WriteHeader;
Const
  H: Array [1..5] of String[70] = (
  '(*******************************************************************)',
  '(*  For use of this file add in your apllication a line            *)',
  '(* {$I%12s}                                                *)',
  '(*                                                                 *)',
  '(*******************************************************************)');

Var
  T: Text;
  S,P,N,E: String;
  i: Byte;
  Pn: Pointer;
Begin
  FSplit(FName, P, N, E);
  S := N + '.inc';
  Pn := @S;
  Assign(T, S);
  ReWrite(T);
  For i := 1 to 5 do
  Begin
    FormatStr(P, H[i], Pn);
    WriteLn(T, P);
  End;
  WriteLn(T, 'Type');
  WriteLn(T, '  P', N, ' = ^T', N);
  WriteLn(T, '  T', N, ' = Record');
  For i := 0 to Fields^.Count - 1 do
    With PField(Fields^.At(i))^ do
    Case What of
      ftDate     : WriteLn(T, '    ', Name, ': String[8];');
      ftCharacter: WriteLn(T, '    ', Name, ': String[', Length, '];');
      ftNumeric  : WriteLn(T, '    ', Name, ': LongInt;');
      ftFloat    : WriteLn(T, '    ', Name, ': Double;');
      ftLogical  : WriteLn(T, '    ', Name, ': Boolean;');
      ftMemo     : WriteLn(T, '(*    ', Name, ': Memo - ignored *)');
    End;
  WriteLn(T, '  End;');
  Close(T);
  MessageBox(^C'Header has beed saved in a file '+S, Nil, mfOkButton+mfInformation);
End;

Function TFieldsListBox.GetText(Item: Integer; MaxLen: Integer): String;
Const
  Form = ' %-3d  %-11s  %-9s  %-4d    %-3d';
Var
  R: PField;
  P: Array[0..4] of LongInt;
  S: String;
Begin
  R := PField(List^.At(Item));
  P[0] := LongInt(Item+1);
  P[1] := LongInt(@R^.Name);
  P[2] := LongInt(@TFieldStr[R^.What]);
  P[3] := LongInt(R^.Length);
  P[4] := LongInt(R^.Decimal);
  FormatStr(S, Form, P);
  GetText := Copy(S, 1, MaxLen);
End;

{
Begin
  PrintStr('Turbo Vision Data Base Files Viewer. Copyright (C) V Kulimov, 1994,95.'#13#10);
}
End.
