Unit
  Zipbuf;

{ Copyright (C) Vacheslav Kulimov, 1996 year.               }
{ FidoNet Address 2:5003/12.12                              }
{ Zip BufStream'd Version. Based on module by Serge Latkin. }
{ Portions copyright 1990-91, By PKWARE Inc.                }
{ All Rights Reserved.                                      }

{$F+,I-}

Interface

Uses
  Dos, Objects;

Const
  ZIP_BINARY = 0; { Compression method }
  ZIP_ASCII  = 1;
  ZIP_BUFSIZ = 35256;
  ZIP_OK     = 0;        { No Errors }
  ZIP_ERFrom = -1;       { Error Opening Input File }
  ZIP_ERTo   = -2;       { Error Opening Output File }
  ZIP_ERROR  = -3;       { other errors (No memory, Bad ZIP file) }

Const
  cfOk      =  0;
  cfErrFrom = -1;
  cfErrTo   = -2;
  cfErrZip  = -3;
  cfErrCopy = -4;

 { CopyFile Modes }

Type
  TCopyFile = ( cfNoZip, cfZip, cfUnZip );

Const
  CompressionType: Word = ZIP_BINARY;

  Function fZip(fromS, toS: PathStr): Integer; {returns Error Code }

  Function fUnZip(fromS, toS: PathStr): Integer;

  Function CopyFile(fFrom, fTo: PathStr; Mode: TCopyFile): Integer;
          { Copy files using TDosStr; optionally Zip-ed }

Implementation

Uses
  MsgBox;

Const
  DictionarySize:  Word = 4096;
  StreamBufferSize: Word = 4096;

Var
  FromFile:        TBufStream;
  FromSize:        LongInt;
  ToFile:          TBufStream;
  Bufff :          Pointer;
  Result:          Integer;

  DiskFullError:   Boolean;
  UnexpectedEOF:   Boolean;

Type
  zipIntFunc  = Function( var Buff; var bSize: Word ): Word;

Function Implode( Read, Write: zipIntFunc; Buf: Pointer;
                  var Ctype: Word; var bSize:Word): Integer;
External;

Function Explode( Read, Write: zipIntFunc; Buf: Pointer):
         Integer;
External;

{$L IMPLODE.OBJ}

Function zipReadData(var Buffer; var BufferSize: Word): Word;
Var
   BytesRead: Word;
Begin
   if FromFile.Status <> stOk then
      UnexpectedEOF := True;

   if BufferSize > FromSize then
   Begin
      FromFile.Read(Buffer, Word(FromSize));
      zipReadData := Word(FromSize)
   End
   else
   Begin
      FromFile.Read(Buffer, BufferSize);
      zipReadData := BufferSize;
      Dec(FromSize, LongInt(BufferSize))
   End;

   if FromFile.Status <> stOk then
   {  - . :-( }
   Begin
      { UnexpectedEOF := True; }
      zipReadData := 0
   End
end;

Function zipWriteData(var Buffer; var BytesRead: Word): Word;
Var
  BytesWritten: Word;
Begin
  ToFile.Write(Buffer, BytesRead);
  if ToFile.Status <> stOk then
  Begin
    zipWriteData := 0;
    DiskFullError := True
  End
  else
    zipWriteData := BytesRead
end;

Function fZip(fromS, toS: PathStr): Integer;
begin
  FromFile.Init(fromS, stOpenRead, StreamBufferSize);
  { Open input file }
  if FromFile.Status <> stOk then
  Begin
    fZip := ZIP_ERFrom;
    FromFile.Done;
    Exit
  End;

  FromSize := FromFile.GetSize;

  ToFile.Init(toS, stCreate, StreamBufferSize);
  { Create output file }
  if ToFile.Status <> stOk then
  Begin
     fZip := ZIP_ERTo;
     FromFile.Done;
     ToFile.Done;
     Exit
  End;

  GetMem(Bufff, ZIP_BUFSIZ);
  if Bufff = Nil then
  Begin
    fZip := ZIP_ERROR;
    FromFile.Done;
    ToFile.Done;
    Exit
  End;

  Result := Implode(zipReadData, zipWriteData, Bufff,
                     CompressionType, DictionarySize);

  FreeMem(Bufff, ZIP_BUFSIZ);
  FromFile.Done;
  ToFile.Done;
  fZip := result;
End;

Function fUnZip(fromS, toS: PathStr): Integer;
Begin
  FromFile.Init(fromS, stOpenRead, StreamBufferSize);
  { Open input file }
  if FromFile.Status <> stOk then
  Begin
    fUnZip := ZIP_ERFrom;
    FromFile.Done;
    Exit
  End;

  FromSize := FromFile.GetSize;

  ToFile.Init(toS, stCreate, StreamBufferSize);
  { Create output file }
  if ToFile.Status <> stOk then
  begin
     fUnZip := ZIP_ERTo;
     FromFile.Done;
     ToFile.Done;
     Exit
  end;

  GetMem( Bufff, ZIP_BUFSIZ );
  if Bufff = Nil then
  begin
     fUnZip := ZIP_ERROR;
     FromFile.Done;
     ToFile.Done;
     Exit
  end;

  Result := Explode(zipReadData, zipWriteData, Bufff);

  FreeMem( Bufff, ZIP_BUFSIZ );
  FromFile.Done;
  ToFile.Done;
  fUnZip := result
end;

Function CopyFile( fFrom, fTo: PathStr; mode: TCopyFile ): Integer;

  Function DosCopyFile(SourceName, TargetName: PathStr): Integer;
  Var
     Source, Target: PBufStream;
  Begin
    CopyFile := 0;
    Source := New(PBufStream, Init(SourceName, stOpenRead, 2048));
    if Source^.Status <> stOk then
    Begin
       DosCopyFile := cfErrFrom;
       Dispose(Source, Done);
       Exit
    End;

    Target := New(PBufStream, Init(TargetName, stCreate, 2048));
    if Target^.Status <> stOk then
    Begin
       DosCopyFile := cfErrTo;
       Dispose(Source, Done);
       Dispose(Target, Done);
       Exit
    End;

    Target^.CopyFrom(Source^, Source^.GetSize);

    if Source^.Status + Target^.Status <> stOk then
       DosCopyFile := cfErrCopy
    else
       DosCopyFile := cfOk;

    Dispose(Source, Done);
    Dispose(Target, Done);
  End;

Var
  io: Integer;

Begin
  io := cfOk;
  DiskFullError := False;
  UnexpectedEOF := False;
  Case mode of
    cfNoZip: io := DosCopyFile(fFrom, fTo);
    cfZip:   io := fZip(fFrom, fTo);
    cfUnZip: io := fUnZip(fFrom, fTo);
  end;
  CopyFile := io;
  if DiskFullError or UnexpectedEOF then
     CopyFile := cfErrCopy;
end;

end.
{ END OF ZIP.PAS }
